@@ -9,7 +9,7 @@ use Carp qw(carp croak);
 use Params::Get;
 use utf8;
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 =head1 NAME
 
@@ -146,22 +146,6 @@ a wide range of regex features.
 
 =over 4
 
-=item * Lookaheads and lookbehinds are not supported
-
-=item * Named groups are not supported
-
-=item * Possessive quantifiers (C<*+>, C<++>) are not supported
-
-=item * Unicode properties (C<\p{}>) are not supported
-
-=item * Some complex nested patterns may not work correctly
-
-=back
-
-=head1 LIMITATIONS
-
-=over 4
-
 =item * Lookaheads and lookbehinds ((?=...), (?!...)) are not supported
 
 =item * Named groups ((?<name>...)) are not supported
@@ -210,7 +194,7 @@ sub new {
 		regex_str => "$regex",
 		length	=> $length || 10,
 		backrefs	=> {},  # Store backreferences
-		named_refs  => {},	   # Store named captures
+		named_refs => {},	 # Store named captures
 	};
 
 	return bless $self, $class;
@@ -239,7 +223,7 @@ sub generate {
 
 	# If smart approach failed, show warning in debug mode
 	if ($ENV{DEBUG_REGEX_GEN} && $@) {
-		warn "Smart generation failed: $@\n";
+		warn "Smart generation failed: $@";
 	}
 
 	# Fall back to brute force with character set matching
@@ -296,7 +280,33 @@ sub generate_smart {
 
 =head2 generate_many($count, $unique)
 
-Generate a number of (possibly) unique strings for the regex
+Generates multiple random strings matching the regex.
+
+    my @strings = $gen->generate_many(10);           # 10 strings (may have duplicates)
+    my @strings = $gen->generate_many(10, 1);        # 10 unique strings
+    my @strings = $gen->generate_many(10, 'unique'); # 10 unique strings
+
+    # Generate until you have 1000 unique codes
+    my $gen = Data::Random::String::Matches->new(qr/[A-Z]{3}\d{4}/);
+    my @codes = $gen->generate_many(1000, 'unique');
+
+Parameters:
+
+=over 4
+
+=item * C<$count> - Number of strings to generate (required, must be positive)
+
+=item * C<$unique> - If true, ensures all generated strings are unique. May return fewer
+than C<$count> strings if uniqueness cannot be achieved within reasonable attempts.
+Accepts any true value (1, 'unique', etc.)
+
+=back
+
+Returns: List of generated strings
+
+Dies: If count is not a positive integer
+
+Warns: If unable to generate the requested number of unique strings
 
 =cut
 
@@ -311,7 +321,7 @@ sub generate_many {
 		# Generate unique strings
 		my %seen;
 		my $attempts = 0;
-		my $max_attempts = $count * 100;  # Reasonable limit
+		my $max_attempts = $count * 100;	# Reasonable limit
 
 		while (keys %seen < $count && $attempts < $max_attempts) {
 			my $str = $self->generate();
@@ -332,6 +342,327 @@ sub generate_many {
 	return @results;
 }
 
+=head2 get_seed()
+
+Gets the random seed for reproducible generation
+
+=cut
+
+sub get_seed {
+	my $self = shift;
+
+	return $self->{seed};
+}
+
+=head2 set_seed($seed)
+
+Sets the random seed for reproducible generation
+
+=cut
+
+sub set_seed {
+	my $self = shift;
+	my $params = Params::Get::get_params('seed', \@_);
+	my $seed = $params->{'seed'};
+
+	croak 'Seed must be defined' unless defined $seed;
+
+	srand($seed);
+	$self->{seed} = $seed;
+
+	return $self;
+}
+
+=head2 suggest_simpler_pattern()
+
+Analyzes patterns and suggests improvements.
+
+  my $suggestion = $gen->suggest_simpler_pattern();
+
+  if ($suggestion) {
+    print "Reason: $suggestion->{reason}\n";
+    print "Better pattern: $suggestion->{pattern}\n" if $suggestion->{pattern};
+    print "Tips:\n";
+    print "  - $_\n" for @{$suggestion->{tips}};
+  }
+
+=cut
+
+sub suggest_simpler_pattern {
+	my $self = $_[0];
+
+	my $pattern = $self->{regex_str};
+	my $info = $self->pattern_info();
+
+	# Check for patterns that are too complex
+	if ($info->{complexity} eq 'very_complex') {
+		return {
+			pattern => undef,
+			reason  => 'Pattern is very complex. Consider breaking it into multiple simpler patterns.',
+			tips    => [
+				'Split alternations into separate generators',
+				'Avoid deeply nested groups',
+				'Use fixed-length patterns when possible',
+			],
+		};
+	}
+
+	# Suggest removing unnecessary backreferences
+	if ($info->{features}{has_backreferences} && $pattern =~ /(\(\w+\)).*\\\d+/) {
+		my $simpler = $pattern;
+		# Can't automatically simplify backreferences, but can suggest
+		return {
+			pattern => undef,
+			reason  => 'Backreferences add complexity. Consider if you really need repeated groups.',
+			tips    => [
+				'If the repeated part doesn\'t need to match, use two separate patterns',
+				'For validation, backreferences are great; for generation, they limit variation',
+			],
+		};
+	}
+
+	# Suggest fixed quantifiers instead of ranges
+	if ($pattern =~ /\{(\d+),(\d+)\}/) {
+		my ($min, $max) = ($1, $2);
+		if ($max - $min > 10) {
+			my $mid = int(($min + $max) / 2);
+			my $simpler = $pattern;
+			$simpler =~ s/\{\d+,\d+\}/\{$mid\}/;
+			return {
+				pattern => $simpler,
+				reason  => "Large quantifier range {$min,$max} creates high variability. Consider fixed length {$mid}.",
+				tips    => [
+					'Fixed lengths are faster to generate',
+					'If you need variety, generate multiple patterns with different fixed lengths',
+				],
+			};
+		}
+	}
+
+	# Suggest limiting alternations
+	if ($info->{features}{has_alternation}) {
+		my @alts = split /\|/, $pattern;
+		if (@alts > 10) {
+			return {
+				pattern => undef,
+				reason  => 'Too many alternations (' . scalar(@alts) . '). Consider splitting into multiple patterns.',
+				tips    => [
+					'Create separate generators for different alternatives',
+					'Group similar patterns together',
+					'Use character classes [abc] instead of (a|b|c)',
+				],
+			};
+		}
+
+		# Check if alternations could be a character class
+		if ($pattern =~ /\(([a-zA-Z])\|([a-zA-Z])\|([a-zA-Z])\)/) {
+			my $chars = join('', $1, $2, $3);
+			my $simpler = $pattern;
+			$simpler =~ s/\([a-zA-Z]\|[a-zA-Z]\|[a-zA-Z]\)/[$chars]/;
+			return {
+				pattern => $simpler,
+				reason  => 'Single-character alternations can be simplified to character classes.',
+				tips    => [
+					'Use [abc] instead of (a|b|c)',
+					'Character classes are faster to process',
+				],
+			};
+		}
+	}
+
+	# Suggest removing lookaheads/lookbehinds for generation
+	if ($info->{features}{has_lookahead} || $info->{features}{has_lookbehind}) {
+		my $simpler = $pattern;
+		$simpler =~ s/\(\?[=!].*?\)//g;   # Remove lookaheads
+		$simpler =~ s/\(\?<[=!].*?\)//g;  # Remove lookbehinds
+
+		if ($simpler ne $pattern) {
+			return {
+				pattern => $simpler,
+				reason  => 'Lookaheads/lookbehinds add complexity but don\'t contribute to generated strings.',
+				tips    => [
+					'Lookaheads are great for validation, not generation',
+					'The simplified pattern generates the same strings',
+				],
+			};
+		}
+	}
+
+	# Check for Unicode when ASCII would work
+	if ($info->{features}{has_unicode} && $pattern =~ /\\p\{L\}/) {
+		my $simpler = $pattern;
+		$simpler =~ s/\\p\{L\}/[A-Za-z]/g;
+		return {
+			pattern => $simpler,
+			reason  => 'Unicode \\p{L} can be simplified to [A-Za-z] if you only need ASCII letters.',
+			tips    => [
+				'ASCII patterns are faster',
+				'Only use Unicode if you need non-ASCII characters',
+			],
+		};
+	}
+
+	# Check for overly long fixed strings
+	if ($pattern =~ /([a-zA-Z]{20,})/) {
+		return {
+			pattern => undef,
+			reason  => 'Pattern contains very long fixed literal strings. Consider if you need such specific patterns.',
+			tips    => [
+				'Use variables instead of long literals',
+				'Break into smaller patterns',
+			],
+		};
+	}
+
+	# Pattern seems reasonable
+	return undef;
+}
+
+=head2 validate($string)
+
+Checks if a string matches the pattern without generating.
+
+  if ($gen->validate('1234')) {
+    print "Valid!\n";
+  }
+
+=cut
+
+sub validate {
+	my $self = shift;
+	my $params = Params::Get::get_params('string', \@_);
+	my $string = $params->{'string'};
+
+	croak('String must be defined') unless defined $string;
+
+	my $regex = $self->{regex};
+	return $string =~ /^$regex$/;
+}
+
+=head2 pattern_info()
+
+Returns detailed information about the pattern.
+
+  my $info = $gen->pattern_info();
+  print "Complexity: $info->{complexity}\n";
+  print "Min length: $info->{min_length}\n";
+  print "Has Unicode: ", $info->{features}{has_unicode} ? "Yes" : "No", "\n";
+
+C<pattern_info> analyzes a regular expression to produce a structured summary of its characteristics,
+including estimated string lengths, detected features, and an overall complexity rating.
+It first calls C<_estimate_length> to heuristically compute the minimum and maximum possible lengths of strings matching the pattern by scanning for literals,
+character classes, and quantifiers.
+It then detects the presence of advanced regex constructions such as alternation, lookahead or lookbehind assertions, named groups, and Unicode properties, storing them in a feature hash.
+Finally, it calculates a rough "complexity" classification based on pattern length and detected features-returning a hash reference that describes the regex's structure, estimated lengths, and complexity level.
+
+=cut
+
+sub pattern_info {
+	my $self = $_[0];
+
+	return $self->{'_pattern_info_cache'} if $self->{'_pattern_info_cache'};
+
+	my $pattern = $self->{'regex_str'};
+
+	# Calculate approximate min/max lengths
+	my ($min_len, $max_len) = $self->_estimate_length($pattern);
+
+	# Detect pattern features
+	my %features = (
+		has_alternation     => ($pattern =~ /\|/ ? 1 : 0),
+		has_backreferences  => ($pattern =~ /(\\[1-9]|\\k<)/ ? 1 : 0),
+		has_unicode         => ($pattern =~ /\\p\{/ ? 1 : 0),
+		has_lookahead       => ($pattern =~ /\(\?[=!]/ ? 1 : 0),
+		has_lookbehind      => ($pattern =~ /\(\?<[=!]/ ? 1 : 0),
+		has_named_groups    => ($pattern =~ /\(\?</ ? 1 : 0),
+		has_possessive      => ($pattern =~ /(?:[+*?]\+|\{\d+(?:,\d*)?\}\+)/ ? 1 : 0),
+	);
+
+	my $info = {
+		pattern             => $pattern,
+		min_length          => $min_len,
+		max_length          => $max_len,
+		estimated_length    => int(($min_len + $max_len) / 2),
+		features            => \%features,
+		complexity          => $self->_calculate_complexity(\%features, $pattern),
+	};
+
+	$self->{'_pattern_info_cache'} = $info;
+
+	return $info;
+}
+
+sub _estimate_length {
+	my ($self, $pattern) = @_;
+
+	# Remove anchors and modifiers
+	$pattern =~ s/^\(\?\^?[iumsx-]*:(.*)\)$/$1/;
+	$pattern =~ s/^\^//;
+	$pattern =~ s/\$//;
+
+	my $min = 0;
+	my $max = 0;
+
+	# Simple heuristic - count fixed characters and quantifiers
+	my $last_was_atom = 0;	# Handle cases like \d{3} where the quantifier modifies the atom count
+	while ($pattern =~ /([^+*?{}\[\]\\])|\\[dwsWDN]|\[([^\]]+)\]|\{(\d+)(?:,(\d+))?\}/g) {
+		if (defined $1 || (defined $2 && $2)) {
+			$min++;
+			$max++;
+			$last_was_atom = 1;
+		} elsif (defined $3) {
+			if ($last_was_atom) {
+				# Replace the last atom’s contribution
+				$min += $3 - 1;
+				$max += (defined $4 ? $4 : $3) - 1;
+				$last_was_atom = 0;
+			} else {
+				# No preceding atom? assume standalone
+				$min += $3;
+				$max += defined $4 ? $4 : $3;
+			}
+		}
+	}
+
+	# Account for +, *, ?
+	my $plus_count = () = $pattern =~ /\+/g;
+	my $star_count = () = $pattern =~ /\*/g;
+	my $question_count = () = $pattern =~ /\?/g;
+
+	$min += $plus_count;  # + means at least 1
+	$max += ($plus_count * 5) + ($star_count * 5);  # Assume max 5 repetitions
+	$min -= $question_count;  # ? makes things optional
+
+	$min = 0 if $min < 0;
+	$max = $min + 50 if $max < $min;  # Ensure max >= min
+
+	return ($min, $max);
+}
+
+sub _calculate_complexity {
+	my ($self, $features, $pattern) = @_;
+
+	my $score = 0;
+
+	# Base complexity from pattern length
+	$score += length($pattern) / 10;
+
+	# Add complexity for features
+	$score += 2 if $features->{has_alternation};
+	$score += 3 if $features->{has_backreferences};
+	$score += 2 if $features->{has_unicode};
+	$score += 2 if $features->{has_lookahead};
+	$score += 2 if $features->{has_lookbehind};
+	$score += 1 if $features->{has_named_groups};
+	$score += 1 if $features->{has_possessive};
+
+	# Classify
+	return 'simple'   if $score < 3;
+	return 'moderate' if $score < 7;
+	return 'complex'  if $score < 12;
+	return 'very_complex';
+}
+
 sub _build_from_pattern {
 	my ($self, $pattern) = @_;
 
@@ -401,11 +732,11 @@ sub _parse_sequence {
 				my ($generated, $new_i) = $self->_handle_quantifier($pattern, $i, sub {
 					my @chars = ('a'..'z', 'A'..'Z', '0'..'9', '_');
 					$chars[int(rand(@chars))];
-				});
+				}, 1);
 				$result .= $generated;
 				$i = $new_i;
 			} elsif ($next eq 's') {
-				my ($generated, $new_i) = $self->_handle_quantifier($pattern, $i, sub { ' ' });
+				my ($generated, $new_i) = $self->_handle_quantifier($pattern, $i, sub { ' ' }, 1);
 				$result .= $generated;
 				$i = $new_i;
 			} elsif ($next eq 'D') {
@@ -576,7 +907,7 @@ sub _handle_quantifier {
 
 	if ($next eq '{') {
 		my $end = index($pattern, '}', $pos + 2);
-		croak "Unmatched '}'" if ($end == -1);
+		croak "Unmatched '{' at position $pos in pattern: $pattern" if ($end == -1);
 		my $quant = substr($pattern, $pos + 2, $end - $pos - 2);
 
 		# Check for possessive after }
@@ -769,7 +1100,7 @@ sub _unicode_property_chars {
 		return ('0' .. '9');
 	} elsif ($prop eq 'Lu' || $prop eq 'Uppercase_Letter') {
 		# Uppercase letters, skip × which is not a letter
-		return ('A' .. 'Z', map { chr($_) } (ord('À') .. ord('Ö'), ord('ø') .. ord('Þ')));
+		return ('A' .. 'Z', map { chr($_) } (ord('À') .. ord('Ö'), ord('Ø') .. ord('Þ')));
 	} elsif ($prop eq 'Ll' || $prop eq 'Lowercase_Letter') {
 		# Lowercase letters, skip ÷ which is not a letter
 		return ('a' .. 'z', map { chr($_) } (ord('à') .. ord('ö'), ord('ø') .. ord('ÿ')));
@@ -814,10 +1145,17 @@ sub create_random_string
 
 Nigel Horne, C<< <njh at nigelhorne.com> >>
 
-=head1 LICENSE
+=head1 SEE ALSO
 
-This is free software; you can redistribute it and/or modify it under
-the same terms as Perl itself.
+=over 4
+
+=item * Test coverage report: L<https://nigelhorne.github.io/Data-Random-String-Matches/coverage/>
+
+=item * L<String::Random>
+
+=item * L<Regexp::Genex>
+
+=back
 
 =head1 LICENCE AND COPYRIGHT
 
